#This module is used to control the ER1 robot through a telent API session. It
#is written in python 2.5, so it is probably not compatible with 3.0+ versions
#of python (it shouldn't be difficult to make it 3.0 compatible).

#TIPS AND TRICKS:  The two functions in this module that are used most for
#controlling the ER1 robot are the evocon and readSocket functions. Commands,
#sent as strings, are entered into the "evocon" function, which then sends it to
#the RCC through a command line API session. Example: evocon('move 12 i') sends
#a command that moves the ER1 12 inches forward (see Chapter 10 of the ER1 manual
#for more info about API commands). After the 'events' command has been sent,
#the readSocket function can be used to read any strings sent from the ER1 (such
#as the recognition of an object). It returns the "event" as a string into the
#main script. The other functions extract information from these "event" strings.
#This script works but is incomplete; more features and/or functions will be added
#in the future for better API control of the ER1 robot.

#This module frequently uses the telnetlib module.  The time and string modules
#are also occasionally used.
import telnetlib
import time
import string

HOST = "localhost"

#This part of the module runs when you import it (It establishes the connection).
print "\nOpening Telnet session to ER1 GUI."
tn = telnetlib.Telnet(HOST, 9000)
print "Opened session.\n"

#Send string commands to the ER1 using this function.  A string command is sent
#to the ER1 by the .write function and any messages it sends back are viewed in
#the .read_until function (usually an 'OK' conformation or an error message).
def evocon(cmd, timeout=1):
    tn.write("%s\n" % cmd)
    a = tn.read_until("\n", timeout)
    if a.count("Error") >= 1 or a.count("error") >= 1:  #Check if there's an error
        print a
    return a

#This function sends the 'events' command and, depending on what 'cmd' equals,
#waits until a certain event happens and exits.  Here are the values that 'cmd'
#may equal:
#'move' ; 'play' ; 'object' (also supply the object's name)
def WaitFor(cmd, name=None):
    timeout = None
    a = ''
    tn.write("%s\n" % 'events')
    time.sleep(.1)
    a = tn.read_until("\n", timeout)
    print a
    if a != '':
        while a != '':
            a = tn.read_until("\n", timeout)
            print a
            #In both cases were you want a move or play to finish, the com-
            #pletion event contains the word "done".  When this event arrives,
            #it stops watching events and the function ends.
            if cmd == 'move':
                if a.count('move done') >= 1:
                    timeout = 1
                    break     #break out of the while loop, so function can end.
            elif cmd == 'play':
                if a.count('play done') >= 1:
                    timeout = 1
                    break
            #When an object is recognized, a string is returned showing the name
            #of the object, how many features have been recognized, how much
            #it has, it's xy coordinates, and it's distance in cm.
            elif cmd == 'object':
                if a.count(name) >= 1:
                    timeout = 1
                    break
    return a

#This function reads the socket for 1 second, or until a string is recieved. Use
#this function for monitoring events in your script(s). Adjust time length that
#works best for your situation.
def readSocket():
    x = tn.read_until("\n", 1)
    return x

#When an specific object is recognized, send that event string into this function.
#The x, y, and distance values of the recognized object are extracted from the
#string and returned as integers and/or floating points for further use in your
#scripts.
def GetVisionData(string):
    start = 0
    if string.count('"') == 2:
        #There are 2 double quote marks in this variable-length string.  Find
        #the indexes of those marks and update the start variable.
        for i in range(2):
           a = string.index('"', start)
           start = a + 1
        start = start + 1
        #There are 2 whitespace characters between the current start index and
        #the x value.  Update the start index twice to get the starting point of
        #The x value.
        for i in range(2):
            a = string.index(' ', start)
            start = a + 1
        #The x value is an integer, so turn the substring containing it into an
        #integer.  The same thing done below is used on extracting the y value.
        x = int(string[start:string.index(' ', start)])
        start = (string.index(' ', start) + 1)
        y = int(string[start:string.index(' ', start)])
        start = (string.index(' ', start) + 1)
        #The distance (z) value is a floating point, that extends to the tenth
        #decimal place (13.6, for example).  The measuring unit is centimeters
        #and it is the last value on the string.
        z = float(string[start:len(string)])
    return x, y, z

#When the "position" command is used, send the resulting event string into this
#function to extract the x and y coordinates as floating points. This allows for
#easy processing and comparisons in your main script. Because the ER1's position
#command does not return the current heading (Evolution Robotics never was able
#to fix it), the angle value is not extracted from the string.
def GetPositionData(string):
    start = 0
    a = string.index(' ', start)         #Whitespaces are counted, as in the 
    start = a + 1                        #"GetVisionData" function.
    x = float(string[start:string.index(' ', start)])
    start = (string.index(' ', start) + 1)
    y = float(string[start:string.index(' ', start)])
    return x, y             #The unit of the coordinates is centimeters.
