#This "module" contains the map of my home.  The A_star algorithm accesses it to
#find a path from one cell to another.  0 means cell is blocked and 1 means cell
#is open.  Because most households cannot be perfectly mapped using equal-sized
#squares, there are special "cases" in certain parts of the house; where instead
#of moving forward to the next cell, it undergoes a series of movements to reach
#the next cell (such as turning 20 deg. and moving 5 ft. forward to reach the
#next cell).

#Scale: 1 cell = 2X2 ft. square.
a = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0]
b = [0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0]
c = [0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0]
d = [0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]
e = [0, 1, 1, 1, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0]
f = [0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0]
g = [0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0]
h = [0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0]
i = [0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0]
j = [0, 1, 1, 1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0]
k = [0, 1, 1, 1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0]
l = [0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
map = [l, k, j, i, h, g, f, e, d, c, b, a]

#-------------------------------------------------------------------------
#This is a separate array for storing parent cells.  The A_star algorithm
#also accesses this.

ap = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
bp = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
cp = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
dp = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
ep = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
fp = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
gp = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
hp = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
ip = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
jp = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
kp = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
lp = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
Pcell = [lp, kp, jp, ip, hp, gp, fp, ep, dp, cp, bp, ap]

#-------------------------------------------------------------------------
#These are the special "case" functions, specific to this map.

#Special case 1:
def case1(m, CommandStrings, forward):
    print "Executing special case 1"
    if m > 1:
        r = m-1
        forward(r, CommandStrings)
    CommandStrings.append('move 20 d')
    CommandStrings.append('move 3 f')
    CommandStrings.append('move -20 d')
    return

def case2(m, CommandStrings, forward):
    print "Executing special case 2"
    if m > 1:
        r = m-1
        forward(r, CommandStrings)
    CommandStrings.append('move -20 d')
    CommandStrings.append('move 3 f')
    CommandStrings.append('move 20 d')
    return

def case3():
    print "Executing special case 3"
    
def case4():
    print "Executing special case 4"

#This function checks if the path following algorithm reaches a special "case".
#If it does, it undergoes the predefined movements to reach the next cell.
def checkCase(x, y, a, m, CommandStrings, forward):
    if len(a) > 0:
       if x == 3 and y == 3 and a[(len(a)-1)] == [3, 4]:
           case2(m, CommandStrings, forward)
           curCellX = 3
           curCellY = 3
           a.append([3, 3])
           ans = 1
       elif x == 3 and y == 4 and a[(len(a)-1)] == [3, 3]:
           case2(m, CommandStrings, forward)
           curCellX = 3
           curCellY = 4
           a.append([3, 4])
           ans = 1
       elif x == 3 and y == 6 and a[(len(a)-1)] == [3, 7]:
           case1(m, CommandStrings, forward)
           curCellX = 3
           curCellY = 6
           a.append([3, 6])
           ans = 1
       elif x == 3 and y == 7 and a[(len(a)-1)] == [3, 6]:
           case1(m, CommandStrings, forward)
           curCellX = 3
           curCellY = 7
           a.append([3, 7])
           ans = 1
       elif x == 6 and y == 3 and a[(len(a)-1)] == [6, 4]:
           case1(m, CommandStrings, forward)
           curCellX = 6
           curCellY = 3
           a.append([6, 3])
           ans = 1
       elif x == 6 and y == 4 and a[(len(a)-1)] == [6, 3]:
           case1(m, CommandStrings, forward)
           curCellX = 6
           curCellY = 4
           a.append([6, 4])
           ans = 1
       elif x == 10 and y == 5 and a[(len(a)-1)] == [11, 5]:
           case3()
           ans = 1
       elif x == 11 and y == 5 and a[(len(a)-1)] == [10, 5]:
           case3()
           ans = 1
       elif x == 12 and y == 7 and a[(len(a)-1)] == [12, 8]:
           case4()
           ans = 1
       elif x == 12 and y == 8 and a[(len(a)-1)] == [12, 7]:
           case4()
           ans = 1
       else:
           ans = 0
    return ans
