#Version 3 is identical to the other versions, but wrote as a module for easy
#integration into scripts. 

#Heuristic constants and formulas for pathfinder:
Dg = 2
Dh = 1

def ClearMap(m):
    for a in m:
        while a.count(4) >= 1:    #Turn all cells equal to 4, equal to 1
            a[a.index(4)] = 1
        while a.count(3) >= 1:    #Turn all cells equal to 3, equal to 1
            a[a.index(3)] = 1
    return

def forward(x, CommandStrings):
    d = x * 24
    CommandStrings.append('move ' + str(d) + ' i')
    return CommandStrings

def rotateCW(x, CommandStrings):
    CommandStrings.append('move -' + str(x) + ' d')
    return CommandStrings

def rotateCCW(x, CommandStrings):
    CommandStrings.append('move ' + str(x) + ' d')
    return CommandStrings

def G(a, sx, sy):
    answerG = Dg * (abs(a[0] - sx) + abs(a[1] - sy))
    return answerG

def H(a, tx, ty):
    Hscore = Dh * (abs(a[0] - tx) + abs(a[1] - ty))
    return Hscore

def F(a, sx, sy, tx, ty):
    Fscore = G(a, sx, sy) + H(a, tx, ty)
    return Fscore

#Other functions used by pathfinder
def ShowPath(map, numpasses, Pcell, startX, startY, targetX, targetY):
    #Display that a path was found and how many loops were made to find it.
    print "Path found! ", "Number of loops =", numpasses

    map[targetY][targetX] = 4        #Target cell is marked as a 4

    curCellX = targetX               #Start with target cell
    curCellY = targetY

    intPathLength = 0           #Path length is 0 (it has not been found yet)

    #Find the parent of the goal cell, then find that cell's parent and so on,
    #until, the current cell is the start cell.  This trail of parent cells is
    #the found path.
    while curCellX != startX or curCellY != startY:
        intPathLength = intPathLength + 1       #Record the length of the path
        P = Pcell[curCellY][curCellX]
        curCellX = P[0]
        curCellY = P[1]
        map[curCellY][curCellX] = 4  #Mark each parent cell with a 4

    print "Path length =", intPathLength

    map.reverse()                    #reverse list "map", so when printed in the
    for i in map:                    #for: loop, it comes out right-side up
        print i

    map.reverse()                    #Return list "map" to normal

#This function searches the specified map to find a path from a given starting
#cell and target cell. The module, which has the map, name; the start cell;
#and the goal cell must be provided. The path found above is then analyzed and
#movement commands are generated, that the ER1 robot can use to follow the path.
def ER1PathMovements(GivenMap, Pcell, start, end, curHeading, checkCase=None):
    
    map = GivenMap
    openlist = []
    closedlist = []
    #Extract x and y-coordinates from start and end cell arguments:
    targetX = end[0]
    targetY = end[1]
    startX = start[0]
    startY = start[1]
    numpasses = 0       #Used to count how many times the while-loop "looped".

    #Set maximum coordinates of map (or else a range error may occur). The max y
    #is the length of map minus 1 and the max x is the length of the first row
    #minus 1 (note that only a rectangular map can be used; the max x value for
    #the whole map is assumed to be the same as the max x value in the first row). 
    maxValX = len(map[0])-1
    maxValY = len(map)-1

    #--------- Pathfinding Algorithm (analyzes map finding a path) -------------

    curCellX = startX                             #start cell is the current cell
    curCellY = startY

    closedlist.append([startX, startY])           #add start cell to closed list

    rest = 0                              #start search.  I set up a dummy variable,
    while not rest:                       #so it would loop.

      numpasses = numpasses + 1       #Counts how many times it has looped.

      NcellX = curCellX
      NcellY = curCellY + 1
      ScellX = curCellX
      ScellY = curCellY - 1
      EcellX = curCellX + 1
      EcellY = curCellY
      WcellX = curCellX -1
      WcellY = curCellY

      #check if north cell is in range, that is, not under 0 or over max value
      if NcellX >= 0 and NcellY >= 0 and NcellX <= maxValX and NcellY <= maxValY:
         #check if it's on the closed list or blocked.  Its blocked if the cell
         #NcellX and NcellY represent is = 0, open is = 1    
         if map[NcellY][NcellX] != 0 and closedlist.count([NcellX, NcellY]) == 0:
             #if it's not on the openlist, add it and set parent.
             if openlist.count([NcellX, NcellY]) == 0:
                 openlist.append([NcellX, NcellY])
                 Pcell[NcellY][NcellX] = [curCellX, curCellY]
             else:
                 #check if path from current cell has a lower G value
                 if G([curCellX, curCellY], startX, startY) + Dg < G([NcellX, NcellY], startX, startY):
                     #if it is, set parent to current cell and recalculate G&F scores
                     #for this cell
                     Pcell[NcellY][NcellX] = [curCellX, curCellY]
                     #G([NcellX, NcellY]) == G([curCellX, curCellY]) + Dg
                     #F([NcellX, NcellY]) == G([NcellX, NcellY]) + H([NcellX, NcellY])

      #repeat above, only for south, east and west cells.
      if ScellX >= 0 and ScellY >= 0 and ScellX <= maxValX and ScellY <= maxValY:
         if map[ScellY][ScellX] != 0 and closedlist.count([ScellX, ScellY]) == 0:
              if openlist.count([ScellX, ScellY]) == 0:
                  openlist.append([ScellX, ScellY])
                  Pcell[ScellY][ScellX] = [curCellX, curCellY]
              else:
                  if G([curCellX, curCellY], startX, startY) + Dg < G([ScellX, ScellY], startX, startY):
                      Pcell[ScellY][ScellX] = [curCellX, curCellY]
                      #G([ScellX, ScellY]) == G([curCellX, curCellY]) + Dg
                      #F([ScellX, ScellY]) == G([ScellX, ScellY]) + H([ScellX, ScellY])

      if EcellX >= 0 and EcellY >= 0 and EcellX <= maxValX and EcellY <= maxValY:
         if map[EcellY][EcellX] != 0 and closedlist.count([EcellX, EcellY]) == 0:
              if openlist.count([EcellX, EcellY]) == 0:
                  openlist.append([EcellX, EcellY])
                  Pcell[EcellY][EcellX] = [curCellX, curCellY]
              else:
                  if G([curCellX, curCellY], startX, startY) + Dg < G([EcellX, EcellY], startX, startY):
                      Pcell[EcellY][EcellX] = [curCellX, curCellY]
                      #G([EcellX, EcellY]) == G([curCellX, curCellY]) + Dg
                      #F([EcellX, EcellY]) == G([EcellX, EcellY]) + H([EcellX, EcellY])

      if WcellX >= 0 and WcellY >= 0 and WcellX <= maxValX and WcellY <= maxValY:
         if map[WcellY][WcellX] != 0 and closedlist.count([WcellX, WcellY]) == 0:
              if openlist.count([WcellX, WcellY]) == 0:
                  openlist.append([WcellX, WcellY])
                  Pcell[WcellY][WcellX] = [curCellX, curCellY]
              else:
                  if G([curCellX, curCellY], startX, startY) + Dg < G([WcellX, WcellY], startX, startY):
                      Pcell[WcellY][WcellX] = [curCellX, curCellY]
                      G([WcellX, WcellY]) == G([curCellX, curCellY]) + Dg
                      F([WcellX, WcellY]) == G([WcellX, WcellY]) + H([WcellX, WcellY])

      if len(openlist) == 0:                         #if openlist is empty, exit
          print "No Solution found"                  #loop; there is no solution.
          break

      smallest = F(openlist[0], startX, startY, targetX, targetY)         #Assume the first member is the smallest.
      for r in openlist:
          sample = F(r, startX, startY, targetX, targetY)
          if sample <= smallest:          #If an item on the list is smaller than
              smallest = sample           #the first item, set it as the new smallest
              NewX = r[0]
              NewY = r[1]           #Keep looping until a smaller value is found

      #add the cell you just selected to the closed list and remove it from the
      #open list.        
      closedlist.append([NewX, NewY])
      openlist.remove([NewX, NewY])

      #Find out if the cell you put in the closed list is the target cell
      #else, mark the current cell, so when you print "map", you can see it
      #was analyzed (analyzed cells are marked with the number 3)
      if NewX == targetX and NewY == targetY:
          ShowPath(map, numpasses, Pcell, startX, startY, targetX, targetY)
          break

      if NewX > -1 and NewY > -1 and NewX < maxValX and NewY < maxValY:
          map[NewY][NewX] = 3

      curCellX = NewX
      curCellY = NewY

    #--------- Movement Generator Section --------------------------------------

    CommandStrings = []                 #List of ER1 movement command strings
                                        #stored here.
    checkedCells = []                   #List of cells that have been checked
                                        #(none at the moment).
    curCellX = startX                   #Start analysis with the start cell.
    curCellY = startY

    moveDistance = 0                       #How far the robot needs to move
                                           #forward.  Set at 0, since this value
                                           #has not been found yet.
    rest = 0                #dummy variable loop again.
    while not rest:
       celltrack = 0        #Kind of a "3-strike, you're out" counter, so the
                            #path's end can be found.
       NcellX = curCellX
       NcellY = curCellY + 1
       ScellX = curCellX
       ScellY = curCellY - 1
       EcellX = curCellX + 1
       EcellY = curCellY
       WcellX = curCellX -1
       WcellY = curCellY

       #Search surrounding cells that are part of the path.  Rotate and move forward
       #as needed, until no more path cells are found (path ends and the ER1 has
       #reached its goal cell).
       if NcellX >= 0 and NcellX <= maxValX and NcellY >= 0 and NcellY <= maxValY:
          if map[NcellY][NcellX] == 4 and checkedCells.count([NcellX, NcellY]) == 0:
              if curHeading != 'north':
                  if curHeading == 'west':
                      if moveDistance > 0:
                         forward(moveDistance, CommandStrings)
                      moveDistance = 0
                      rotateCW(90, CommandStrings)
                      curHeading = 'north'
                      print "turned 90 deg CW from west to north"
                  elif curHeading == 'east':
                      if moveDistance > 0:
                         forward(moveDistance, CommandStrings)
                      moveDistance = 0
                      rotateCCW(90, CommandStrings)
                      curHeading = 'north'
                      print "turned 90 deg CCW from east to north"
                  else:
                      if moveDistance > 0:
                         forward(moveDistance, CommandStrings)
                      moveDistance = 0
                      rotateCW(180, CommandStrings)
                      curHeading = 'north'
                      print "turned 180 deg CW from south to north"
              else:
                 moveDistance = moveDistance + 1
                 checkedCells.append([curCellX, curCellY])
                 curCellX = NcellX
                 curCellY = NcellY
                 if checkCase != None:
                    if checkCase(curCellX, curCellY, checkedCells, moveDistance, CommandStrings, forward) == 1:
                        moveDistance = 0
          else:
              celltrack = celltrack + 1
       else:
           celltrack = celltrack + 1

       if ScellX >= 0 and ScellX <= maxValX and ScellY >= 0 and ScellY <= maxValY:
          if map[ScellY][ScellX] == 4 and checkedCells.count([ScellX, ScellY]) == 0:
              if curHeading != 'south':
                  if curHeading == 'east':
                      if moveDistance > 0:
                         forward(moveDistance, CommandStrings)
                      moveDistance = 0
                      rotateCW(90, CommandStrings)
                      curHeading = 'south'
                      print "turned 90 deg CW from East to south"
                  elif curHeading == 'west':
                      if moveDistance > 0:
                         forward(moveDistance, CommandStrings)
                      moveDistance = 0
                      rotateCCW(90, CommandStrings)
                      curHeading = 'south'
                      print "turned 90 deg CCW from west to south"
                  else:
                      if moveDistance > 0:
                         forward(moveDistance, CommandStrings)
                      moveDistance = 0
                      rotateCW(180, CommandStrings)
                      curHeading = 'south'
                      print "turned 180 CW deg from north to south"
              else:
                  moveDistance = moveDistance + 1
                  checkedCells.append([curCellX, curCellY])
                  curCellX = ScellX
                  curCellY = ScellY
                  if checkCase != None:
                     if checkCase(curCellX, curCellY, checkedCells, moveDistance, CommandStrings, forward) == 1:
                         moveDistance = 0
          else:
              celltrack = celltrack + 1
       else:
           celltrack = celltrack + 1

       if EcellX >= 0 and EcellX <= maxValX and EcellY >= 0 and EcellY <= maxValY:
          if map[EcellY][EcellX] == 4 and checkedCells.count([EcellX, EcellY]) == 0:
              if curHeading != 'east':
                  if curHeading == 'north':
                      if moveDistance > 0:
                          forward(moveDistance, CommandStrings)
                      moveDistance = 0
                      rotateCW(90, CommandStrings)
                      curHeading = 'east'
                      print "turned 90 deg CW from north to east"
                  elif curHeading == 'south':
                      if moveDistance > 0:
                         forward(moveDistance, CommandStrings)
                      moveDistance = 0
                      rotateCCW(90, CommandStrings)
                      curHeading = 'east'
                      print "turned 90 deg CCW from south to east"
                  else:
                      if moveDistance > 0:
                         forward(moveDistance, CommandStrings)
                      moveDistance = 0
                      rotateCW(180, CommandStrings)
                      curHeading = 'east'
                      print "turned 180 deg CW from west to east"
              else:
                  moveDistance = moveDistance + 1
                  checkedCells.append([curCellX, curCellY])
                  curCellX = EcellX
                  curCellY = EcellY
                  if checkCase != None:
                     if checkCase(curCellX, curCellY, checkedCells, moveDistance, CommandStrings, forward) == 1:
                         moveDistance = 0
          else:
              celltrack = celltrack + 1
       else:
           celltrack = celltrack + 1

       if WcellX >= 0 and WcellX <= maxValX and WcellY >= 0 and WcellY <= maxValY:
          if map[WcellY][WcellX] == 4 and checkedCells.count([WcellX, WcellY]) == 0:
              if curHeading != 'west':
                  if curHeading == 'south':
                      if moveDistance > 0:
                         forward(moveDistance, CommandStrings)
                      moveDistance = 0
                      rotateCW(90, CommandStrings)
                      curHeading = 'west'
                      print "turned 90 deg CW from south to west"
                  elif curHeading == 'north':
                      if moveDistance > 0:
                         forward(moveDistance, CommandStrings)
                      moveDistance = 0
                      rotateCCW(90, CommandStrings)
                      curHeading = 'west'
                      print "turned 90 deg CCW from north to west"
                  else:
                      if moveDistance > 0:
                         forward(moveDistance, CommandStrings)
                      moveDistance = 0
                      rotateCW(180, CommandStrings)
                      curHeading = 'west'
                      print "turned 180 deg CW from east to west"
              else:
                  moveDistance = moveDistance + 1
                  checkedCells.append([curCellX, curCellY])
                  curCellX = WcellX
                  curCellY = WcellY
                  if checkCase != None:
                     if checkCase(curCellX, curCellY, checkedCells, moveDistance, CommandStrings, forward) == 1:
                         moveDistance = 0
          else:
              celltrack = celltrack + 1
       else:
           celltrack = celltrack + 1

       if celltrack == 4:
           if checkCase != None:
              checkCase(curCellX, curCellY, checkedCells, moveDistance, CommandStrings, forward)
           if moveDistance > 0:
              forward(moveDistance, CommandStrings)
           print "Reached end of path"
           ClearMap(map)
           #Save curHeading to the CommandStrings list so it can be utilized
           #when returned at the end of the function (by using the .pop() feature).
           CommandStrings.append(curHeading)
           break
    #Return list of ER1 movement strings, which can then be used to move the ER1
    #from the start cell to the goal cell in the main script.
    return CommandStrings
